@@ -5,15 +5,12 @@
 # Also due to the short life span of reloc tree, save the as dump example for
 # later usage.
 
-source "$TOP/tests/common"
+source "$TEST_TOP/common"
 
 check_prereq btrfs
 
 check_image() {
-	local image
-
-	image=$1
-	run_check "$TOP/btrfs" check "$image"
+	run_check "$TOP/btrfs" check "$1"
 }
 
 check_all_images
